package de.hdm_stuttgart.mi.sd1.task1;

import de.hdm_stuttgart.mi.sd1.task1.model.Employee;

/**
 * <p>A company's salaries.</p>
 */
public class _4_Salary {

    /**
     * <p>Limiting salaries in case of bad financial times.</p>
     *
     * <p>An employee's regular salary is being defined by {@link Employee#getSalary()}. Unfortunately sometimes
     * the sum of all salaries exceeds the company's financial capabilities.</p>
     *
     * <p>We consider an example of two employees having salaries of 2000€ and 4000€ respectively. The sum of their
     * salaries being 6000 is supposed to exceed the company's limit of 3000 for all salaries. Both salaries will
     * be cut by the same fraction of <code style="color:red">3000. / 6000 == 0.5</code> or 50%
     * using {@link Math#round(double)} if so required. The employees' effective salaries will thus be cut to
     * <code>2000 * <span style="color:red">0.5</span> == 1000</code> and
     * <code>4000 * <span style="color:red">0.5</span> == 2000</code> respectively.</p>
     *
     * <p>Limiting salaries is being achieved by setting the {@link Employee#setSalaryFraction(double)} property
     * accordingly.</p>
     *
     * @param overallSalarySum The company's limit for the sum of all salary payments.
     * @param employees An array of employees. Depending on <code>limit</code> their effective salaries may be reduced.
     *
     * <section class="implementationHints">
     *    <h4>Hints:</h4>
     *    <ol>
     *        <li>You more than likely want to implement {@link Employee#getSalary()} before working on this one.</li>
     *
     *        <li>As you notice this method does not return anything. Your solution is supposed to modify the
     *        {@link Employee} instances' salaries being passed by the <code>Employee[]</code> array.</li>
     *
     *        <li>Regarding both fractions and type conversions ({@link Math#round(double)} returns <code>long</code>)
     *                  your solution may require casts.</li>
     *    </ol>
     * </section>
     */
    static public void limitSalariesIfRequired(final int overallSalarySum, final Employee[] employees) {

        Employee.setSalaryFraction(1.0);
        int salariesSum = 0;
        for (final Employee employee: employees) {
            salariesSum += employee.getSalary();
        }
        // Overall payment has to be limited?
        if (overallSalarySum < salariesSum) {
            Employee.setSalaryFraction(overallSalarySum * 1.0 /salariesSum);
        }
    }

    private _4_Salary(){/* Ignore me: Suppressing default constructor javadoc generation:
                                    javadoc does not include private constructors by default */}
}
