package de.hdm_stuttgart.mi.sd1.task2;

/**
 * <p>This class provides information about a given sentence.</p>
 *
 * <p>The following example computes a sentence's:</p>
 *
 * <ul>
 *     <li>Number of digits</li>
 *     <li>Number of letters</li>
 *     <li>Number of uppercase letters</li>
 *     <li>Number of lowercase letters</li>
 * </ul>
 *
 * <table class="goikTableDefaults">
 *     <caption>Counting a string's components.</caption>
 *     <tbody>
 *         <tr>
 *             <th>Code</th>
 *             <th>Result</th>
 *         </tr>
 *         <tr>
 *             <td>
 *                 <pre><code class="java"> final Sentence sentence = new Sentence("99 Java exercises.");
 *
 *  System.out.println("Number of digits: " + sentence.getNumberOfDigits());
 *  System.out.println("Number of letters: " + sentence.getNumberOfLetters());
 *
 *  // true and false distinguishes between upper- ond lowercase letter count.
 *  System.out.println("Number of uppercase letters: " + sentence.getNumberOfLetters(true)); // Uppercase
 *  System.out.println("Number of lowercase letters: " + sentence.getNumberOfLetters(false));// Lowercase</code></pre>
 *             </td>
 *             <td>
 *                 <pre><code class="nohighlight"> Number of digits: 2
 * Number of letters: 13
 * Number of uppercase letters: 1
 * Number of lowercase letters: 12</code></pre>
 *             </td>
 *         </tr>
 *     </tbody>
 *
 * </table>
 *
 * <section class="implementationHints">
 *    <h2>Hints:</h2>
 *
 *    <ul>
 *        <li><p>The class is yet unimplemented. The above code snippet provides a clue how implement {@link Sentence}
 *              to satisfy the corresponding unit tests.</p></li>
 *        <li>
 *            <p>The following methods / variables may be helpful:</p>
 *            <ul>
 *                <li>{@link Character#isDigit(char)}</li>
 *                <li>{@link Character#isLetter(char)}</li>
 *                <li>{@link Character#isUpperCase(char)}</li>
 *                <li>{@link Character#isLowerCase(char)}</li>
 *                <li>{@link Character#getType(char)}</li>
 *                <li>{@link Character#DECIMAL_DIGIT_NUMBER} and friends</li>
 *            </ul>
 *        </li>
 *    </ul>
 *
 * </section>
 */
public class Sentence {

  final private int numberOfUppercaseLetters, numberOfLowercaseLetters, numberOfDigits;

  /**
   * Extracting a string's components
   * @param sentence The string to be handled.
   */
  public Sentence(final String sentence) {

    // Temporary »shadow« values being required because of related final instance values .
    int numberOfUppercaseLetters = 0, numberOfLowercaseLetters = 0, numberOfDigits = 0;

    for (final char c: sentence.toCharArray()) {
      switch (Character.getType(c)) {
        case Character.DECIMAL_DIGIT_NUMBER:
          numberOfDigits++;
          break;
        case Character.UPPERCASE_LETTER:
          numberOfUppercaseLetters++;
          break;
        case Character.LOWERCASE_LETTER:
          numberOfLowercaseLetters++;
          break;
      }
    }
    this.numberOfUppercaseLetters = numberOfUppercaseLetters;
    this.numberOfLowercaseLetters = numberOfLowercaseLetters;
    this.numberOfDigits = numberOfDigits;
  }

  /**
   * String's letters.
   * @return The string's letter count.
   */
  public int getNumberOfLetters() {
    return numberOfUppercaseLetters + numberOfLowercaseLetters;
  }

  /**
   * The string's upper- or lowercase letter count respectively.
   *
   * @param uppercase Distinguish between lower- and uppercase letter count.
   * @return Uppercase letter count if <code>uppercase == true</code>, lowercase letter count otherwise.
   */
  public int getNumberOfLetters(final boolean uppercase) {
    return uppercase? numberOfUppercaseLetters : numberOfLowercaseLetters;
  }

  /**
   * The string's number of digits
   * @return The string's digit count.
   */
  public int getNumberOfDigits() {
    return numberOfDigits;
  }
}
