package de.hdm_stuttgart.mi.sd1.test.task1;

import de.hdm_stuttgart.mi.exam.unitmarking.ExaminationTestDefaults;
import de.hdm_stuttgart.mi.exam.unitmarking.Marking;
import de.hdm_stuttgart.mi.sd1.task1.model.Employee;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import static de.hdm_stuttgart.mi.sd1.task1._4_Salary.limitSalariesIfRequired;

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class Test_4_Salary extends ExaminationTestDefaults {

    static public final String DEBUG_TEST_HINT =
            "Test error: Wanna set a breakpoint next line to start debugging?";

    public static final Logger log = LogManager.getLogger();


    @Test @Marking(points = 4)
    public void test_100() {

        final Employee e = new Employee(0,0);
        Assert.assertEquals(2000, e.getSalary());
    }

    @Test @Marking(points = 4)
    public void test_500_empty() {
        limitSalariesIfRequired(30000, new Employee[0]);// Meaningless
        Assert.assertEquals(2000, new Employee(0,0).getSalary());
        limitSalariesIfRequired(0, new Employee[0]); // Meaningless
        Assert.assertEquals(2000, new Employee(0,0).getSalary());
    }

    @Test
    public void test_520_single() {
        Employee[] employees = {new Employee(2,10)};
        Assert.assertEquals(2900, employees[0].getSalary());

        {// Companies payment limit is well above sum of all salaries
            limitSalariesIfRequired(3000, employees);
            Assert.assertEquals(2900, employees[0].getSalary());
        }

        {// Companies payment limit matches sum of all salaries
            limitSalariesIfRequired(2900, employees);
            Assert.assertEquals(2900, employees[0].getSalary());
        }

        for (short i = 0; i < 2900; i++) {// Companies payment limit beneath sum of all salaries
            limitSalariesIfRequired( i, employees);
            try {
                Assert.assertEquals("Payment limit " + i + ":", i, employees[0].getSalary());
            } catch (final AssertionError e) {
                log.error(DEBUG_TEST_HINT);
                limitSalariesIfRequired( i, employees);
                Assert.assertEquals("Payment limit " + i + ":", i,
                        employees[0].getSalary());
            }
        }
    }

    @Test
    public void test_540_standard() {

        final Employee[] employees = new Employee[10];
        for (byte i = 0; i < employees.length; i++) {
            employees[i] = new Employee(0, 0);
        }

        {// Companies payment limit is well above sum of all salaries
            limitSalariesIfRequired(30000, employees);
            for (final Employee e : employees) {
                Assert.assertEquals(2000, e.getSalary());
            }
        }

        {// Companies payment limit matches sum of all salaries
            limitSalariesIfRequired(20000, employees);
            for (final Employee e : employees) {
                Assert.assertEquals(2000, e.getSalary());
            }
        }

        {// Companies payment limit slightly below sum of all salaries.
            limitSalariesIfRequired(19999, employees);
            for (final Employee e : employees) {
                Assert.assertEquals(2000, e.getSalary());
            }
        }

        {// Companies payment limit just half of all salaries' sum
            limitSalariesIfRequired(10000, employees);
            for (final Employee e : employees) {
                Assert.assertEquals(1000, e.getSalary());
            }
        }

        {// Companies payment limit just quarter of all salaries' sum
            limitSalariesIfRequired(5000, employees);
            for (final Employee e : employees) {
                Assert.assertEquals(500, e.getSalary());
            }
        }
        {// Bad year: No salaries
            limitSalariesIfRequired(0, employees);
            for (final Employee e : employees) {
                Assert.assertEquals(0, e.getSalary());
            }
        }
    }

    @Test
    public void test_600_mixed() {

        final Employee[] employees = new Employee[10];

        for (byte i = 0; i < employees.length; i++) {
            employees[i] = new Employee(i, 2 * i);
        }

        {// Companies payment limit is well above sum of all salaries
            limitSalariesIfRequired(40000, employees);
            for (byte i = 0; i < employees.length; i++) {
                Assert.assertEquals("Employee index " + i + ":", 2000 + 300 * i, employees[i].getSalary());
            }
        }

        {// Companies payment limit matches sum of all salaries
            limitSalariesIfRequired(33500, employees);
            for (byte i = 0; i < employees.length; i++) {
                Assert.assertEquals("Employee index " + i + ":", 2000 + 300 * i, employees[i].getSalary());
            }
        }

        {// Companies payment limit just half of sum of all salaries
            limitSalariesIfRequired(33500 / 2, employees);
            for (byte i = 0; i < employees.length; i++) {
                Assert.assertEquals("Employee index " + i + ":", 1000 + 150 * i, employees[i].getSalary());
            }
        }

        {// Companies payment limit just 1% of sum of all salaries
            limitSalariesIfRequired(33500 / 100, employees);
            for (byte i = 0; i < employees.length; i++) {
                Assert.assertEquals("Employee index " + i + ":", 20 + 3 * i, employees[i].getSalary());
            }
        }

        {// Bad year: No salaries
            limitSalariesIfRequired(0, employees);
            for (byte i = 0; i < employees.length; i++) {
                Assert.assertEquals("Employee index " + i + ":", 0, employees[i].getSalary());
            }
        }
    }
}
